#include "defbf533.h"

.extern _config_latches;
.global _main;

#define OFFSET_( x ) ((x) & 0x0000FFFF)
#define UPPER_( x ) (((x) >> 16) & 0x0000FFFF)
#define LOWER_( x ) ((x) & 0x0000FFFF)

#define SYSMMR_BASE 0xFFC00000
#define COREMMR_BASE 0xFFE00000

.section program;

_main:

	[--SP] = ASTAT;
	[--SP] = RETS;
	[--SP] = (r7:0);
	[--SP] = (p5:0);
	
	P0.H = UPPER_(COREMMR_BASE);			// P0 Points to the beginning of CORE MMR Space
	P0.L = LOWER_(COREMMR_BASE);

	P1.H = UPPER_(SYSMMR_BASE);				// P1 Points to the beginning of SYSTEM MMR Space
	P1.L = LOWER_(SYSMMR_BASE);

	R0 = 0x10(z);
	W[P1+OFFSET_(SPI_BAUD)] = R0;
	
	call.x _config_latches;  // (JWS) Added to configure outboard latches in PMP


// (JWS) Added PLL change as PMP is defaulting to 122MHz CCLK and 24MHz SCLK.

PLL_change:

	P0.L = lo(SIC_IWR);
	P0.H = hi(SIC_IWR);
	R1 = [P0];
	R0 = 0x1; 
	[P0] = R0;
	
	P1.L = lo(PLL_CTL);
	P1.H = hi(PLL_CTL);
	R0.L = 0x3000; 		// Default for BF533 is 0x1400 = 10X CLKIN
				// CLKIN = 12.288MHz for the PMP
				// PLL_CTL = 0[xxx|xxx]0|0000|0000 where xxxxxx is the MSEL value
				// MSEL = 011000 = 24X = 24 X 12.288 = 294.912 MHz
				// PLL_CTL = 0x3000


	w[P1] = R0;
	ssync;
	
	idle;
	
	[P0] = R1;
	
	P1.L = lo(PLL_DIV);
	P1.H = hi(PLL_DIV);
	R0.L = 0x3; 		// Default for BF533 is 0x0005 = CCLK/5
				// CCLK/3 = 294.912/3 = 98.304


	w[P1] = R0;
	
/******************************/



	
/* (JWS) Done in config_latches - set for mSDRAM
CONFIG_SDRAM:
	R0 = 0x0817(Z);					
	W[P1+OFFSET_(EBIU_SDRRC)] = R0;			//SDRAM Refresh Rate Control Register
	SSYNC;	

	R0 = 0x0013(Z);
	W[P1+OFFSET_(EBIU_SDBCTL)] = R0;		//SDRAM Memory Bank Control Register
	SSYNC;		

	R0.L = 0x998D;
	R0.H = 0x0091;
	[P1+OFFSET_(EBIU_SDGCTL)] = R0;			//SDRAM Memory Global Control Register
	SSYNC;
*/	
CHECK_KEY:
    P2.H = 0x0000;
    P2.L = 0x3FEC;							//Key is located at address 0x3FEC
    R0.H = 0x1234;
    R0.L = 0x5678;
    R1 = [P2++];
    SSYNC;
    CC = R0 == R1;
    IF CC JUMP RESTORE;						//If the key matches, don't do a full boot

FULL_BOOT:
	(p5:0) = [SP++];
	(r7:0) = [SP++];
	RETS = [SP++];
	ASTAT = [SP++];
	RTS;									//If the key doesn't match, return back to the bootrom for a full boot

RESTORE:
//Restore Stack Pointer
	SP = [P2++];
	
//Restore Frame Pointer
	FP = [P2++];

//Restore L1 Memory via DMA
    R0 = [P2++];							//P2 will point to the Return Address after this update
    [P1+OFFSET_(MDMA_S0_CURR_DESC_PTR)] = R0;
    R0 += 0x20;
    [P1+OFFSET_(MDMA_D0_CURR_DESC_PTR)] = R0;
    R0 = (FLOW-0x3000)|(NDSIZE-0x400)|RESTART|WDSIZE_32|DMAEN(z);
    W[P1+OFFSET_(MDMA_S0_CONFIG)] = R0;
    R0 = (FLOW-0x3000)|(NDSIZE-0x400)|RESTART|WDSIZE_32|WNR|DMAEN(z);
    W[P1+OFFSET_(MDMA_D0_CONFIG)] = R0;
    
//Restore Scratchpad Memory via Core reads/writes
	P3.H = 0x0001;							//P3 = SDRAM Memory
	P3.L = 0xC000;
	P4.H = 0xFFB0;							//P4 = Scratchpad Memory
	P4.L = 0x0000;
	P5.H = 0x0000;							//Restore 4096 bytes
	P5.L = 0x0400;
	LSETUP(L3_TO_SCRATCH_begin, L3_TO_SCRATCH_end) LC0 = P5;
L3_TO_SCRATCH_begin:	R0 = [P3++];
L3_TO_SCRATCH_end:		[P4++] = R0;

//Restore EVT Table via Core reads/writes
	P4.H = 0xFFE0;							//P4 = Event Vector Table
	P4.L = 0x2000;
	P5.H = 0x0000;							//Restore 16 entries
	P5.L = 0x0010;
	LSETUP(L3_TO_EVT_begin, L3_TO_EVT_end) LC0 = P5;
L3_TO_EVT_begin:	R0 = [P3++];
L3_TO_EVT_end:		[P4++] = R0;

//Restore IMASK
	P4.H = 0xFFE0;
	P4.L = 0x2104;
	R0 = [P3++];
	[P4] = R0;

//Wait until L1 Memory DMA is complete
TST_RUN_BIT:
    R0 = W[P1+OFFSET_(MDMA_D0_IRQ_STATUS)](z);
    CC = BITTST(R0,3);
    IF CC JUMP TST_RUN_BIT;					//Wait til DMA is complete
    
    
/**** For profiling purposes ****/

/*  (JWS) This will probably screw up the SDRAM tristate buffer
	Unless we or in the other bits
	
	R0 = 0x0080;							//Enable PF7 as output
	W[P1+OFFSET_(FIO_DIR)] = R0;
	R0 = 0x0080;							//Set PF7 high
	W[P1+OFFSET_(FIO_FLAG_S)] = R0;
	
*/
/********************************/

	P5 = [P2];
    JUMP(P5);
    
_main.end:
